@@ -3,8 +3,8 @@ __all__ = [
     'load_fits',
     'load_cfg',
     'load_lines_log',
-    'save_line_log'
-]
+    'save_line_log',
+    'save_param_maps']
 
 import os
 import configparser
@@ -13,7 +13,7 @@ import numpy as np
 import pandas as pd
 import pylatex
 
-from sys import exit
+from sys import exit, stdout
 from pathlib import Path
 from functools import partial
 from collections import Sequence
@@ -170,7 +170,7 @@ DARK_PLOT = {'figure.figsize': (14, 7),
 STANDARD_AXES = {'xlabel': r'Wavelength $(\AA)$', 'ylabel': r'Flux $(erg\,cm^{-2} s^{-1} \AA^{-1})$'}
 
 STRINGCONFKEYS = ['sampler', 'reddenig_curve', 'norm_line_label', 'norm_line_pynebCode']
-GLOBAL_LOCAL_GROUPS = ['_line_fitting', '_chemical_model']
+GLOBAL_LOCAL_GROUPS = ['line_fitting', 'chemical_model'] # TODO not implemented
 
 FLUX_TEX_TABLE_HEADERS = [r'$Transition$', '$EW(\AA)$', '$F(\lambda)$', '$I(\lambda)$']
 FLUX_TXT_TABLE_HEADERS = [r'$Transition$', 'EW', 'EW_error', 'F(lambda)', 'F(lambda)_error', 'I(lambda)', 'I(lambda)_error']
@@ -188,21 +188,8 @@ def check_numeric_Value(s):
         return s
 
 
-# Function to import configparser
-def importConfigFile(config_path):
-    # Check if file exists
-    if os.path.isfile(config_path):
-        cfg = configparser.ConfigParser()
-        cfg.optionxform = str
-        cfg.read(config_path)
-    else:
-        exit(f"--WARNING: Configuration file {config_path} was not found. Exiting program")
-
-    return cfg
-
-
 # Function to map a string to its variable-type
-def formatStringEntry(entry_value, key_label, section_label='', float_format=None, nan_format='nan'):
+def format_option_value(entry_value, key_label, section_label='', float_format=None, nan_format='nan'):
     output_variable = None
 
     # None variable
@@ -231,7 +218,9 @@ def formatStringEntry(entry_value, key_label, section_label='', float_format=Non
             else:
                 output_variable = check_numeric_Value(entry_value)
 
-    # Arrays (The last boolean overrides the parameters # TODO unstable in case of one item lists
+    # Arrays (The last boolean overrides the parameters
+    # TODO keys with array are always converted to numpy array even if just one
+
     elif ',' in entry_value:
 
         # Specia cases conversion
@@ -285,46 +274,101 @@ def formatStringEntry(entry_value, key_label, section_label='', float_format=Non
     return output_variable
 
 
-# Function to import SpecSyzer configuration file #TODO repeated
-def load_cfg(filepath, objList_check=False):
+# Function to import SpecSyzer configuration file
+def load_cfg(file_address, obj_section=None, mask_section=None, def_cfg_sec='line_fitting'):
+
+    """
+
+    This function reads a configuration file with the `standard ini format <https://en.wikipedia.org/wiki/INI_file>`_. Please
+    check the ``.format_option_value`` function for the special keywords conversions done by LiMe.
+
+    If the user provides a list of objects (via the ``obj_section`` parameter) this function will update the object line fitting
+    configuration to include the default configuration. If there are shared entries the object configuration takes preference.
+    The object section must have follow the "objectName_line_fitting" notation, where the "objectName" is obtained from
+    the object list.
+
+    If the user provides a list of masks (via the ``mask_section`` parameter) this function will update the spaxel line fitting
+    configuration to include the mask configuration. If there are shared entries the spaxel configuration takes preference.
+    The spaxel  section must have follow the "idxY-idxX_line_fitting" notation, where "idxY-idxX" are the y and x indices
+    of the masked spaxel obtained from the mask.
+
+    .. attention::
+        For the right formatting of the line fitting configuration entries the user must include the "line_fitting" string
+        in the file configuration section name. For example:
+
+    .. code-block::
+
+        [default_line_fitting]
+        H1_6563A_b = H1_6563A-N1_6584A-N1_6548A
+
+        [IZwicky18_line_fitting]
+        O2_3726A_m = O2_3726A-O2_3729A
+
+        [123-84_line_fitting]
+        H1_6563A_b = H1_6563A-N1_6584A
+
+    :param file_address: configuration file location
+    :type file_address: str or ~pathlib.Path
+
+    :param obj_section: the section:option location for the list of objects, e.g. {'sample data': 'obj_list'}
+    :type obj_section: dict, optional
+
+    :param mask_section: the section:option location of the spatial masks, e.g. {'sample data': 'obj_mask_list'}
+    :type mask_section: dict, optional
+
+    :param def_cfg_sec: the section(s) with the line fitting configuration, e.g. 'default_line_fitting'
+    :type def_cfg_sec: str or list, optional
+
+    :return: Parsed configuration data
+    :rtype: dict
+
+    """
 
     # Open the file
-    if Path(filepath).is_file():
-        cfg = importConfigFile(filepath)
-        # TODO keys with array are always converted to numpy array even if just one
+    if Path(file_address).is_file():
+        cfg = configparser.ConfigParser()
+        cfg.optionxform = str
+        cfg.read(file_address)
     else:
-        exit(f'-ERROR Configuration file not found at:\n{filepath}')
-
-    confDict = {}
+        exit(f'-ERROR Configuration file not found at:\n{file_address}')
 
+    # Convert the configuration entries from the string format if possible
+    cfg_lime = {}
     for section in cfg.sections():
-        confDict[section] = {}
+        cfg_lime[section] = {}
         for option_key in cfg.options(section):
             option_value = cfg[section][option_key]
-            confDict[section][option_key] = formatStringEntry(option_value, option_key, section)
+            cfg_lime[section][option_key] = format_option_value(option_value, option_key, section)
 
-    if objList_check is True:
+    # Update the object line fitting sections if provided by user
+    if obj_section is not None:
 
-        assert 'file_information' in confDict, '- No file_information section in configuration file'
-        assert 'object_list' in confDict['file_information'], '- No object_list option in configuration file'
-        objList = confDict['file_information']['object_list']
+        for sec_objs, opt_objs in obj_section.items():
 
-        # Combine sample with obj properties if available
-        if objList is not None:
-            for key_group in GLOBAL_LOCAL_GROUPS:
-                global_group = f'default{key_group}'
-                if global_group in confDict:
-                    for objname in objList:
-                        local_group = f'{objname}{key_group}'
-                        dict_global = copy.deepcopy(confDict[global_group])
-                        if local_group in confDict:
-                            dict_global.update(confDict[local_group])
-                        confDict[local_group] = dict_global
+            # Get the list of objects
+            assert sec_objs in cfg_lime, f'- ERROR: No {sec_objs} section in file {file_address}'
+            assert opt_objs in cfg_lime[sec_objs], f'- ERROR: No {opt_objs} option in section {sec_objs} in file {file_address}'
+            objList = cfg_lime[sec_objs][opt_objs]
 
-    return confDict
+            # Get the default configuration
+            assert def_cfg_sec in cfg_lime, f'- ERROR: No {def_cfg_sec} section in file {file_address}'
+            global_cfg = cfg_lime[def_cfg_sec]
+
+            # Loop through the objects
+            for obj in objList:
+                global_dict = copy.deepcopy(global_cfg)
+
+                local_label = f'{obj}_line_fitting'
+                local_dict = cfg_lime[local_label] if local_label in cfg_lime else {}
+
+                # Local configuration overwriting global
+                global_dict.update(local_dict)
+                cfg_lime[local_label] = global_dict
+
+    return cfg_lime
 
 
-def numberStringFormat(value, cifras = 4):
+def numberStringFormat(value, cifras=4):
     if value > 0.001:
         newFormat = f'{value:.{cifras}f}'
     else:
@@ -387,14 +431,26 @@ def format_for_table(entry, rounddig=4, rounddig_er=2, scientific_notation=False
 
 
 def load_lines_log(lineslog_address, ext=None):
+
     """
-    This function attemps several approaches to import a lines log from a sheet or text file lines as a pandas
-    dataframe
-    :param lineslog_address: String with the location of the input lines log file
-    :return lineslogDF: Dataframe with line labels as index and default column headers (wavelength, w1 to w6)
+    This function reads a lines log file as a pandas dataframe. In the case of .fits or excel files the user must specify
+    the extension.
+
+    :param lineslog_address: Address of the configuration file. The function will stop if the file is not found
+    :type lineslog_address: str
+    :type lineslog_address: Path
+
+    :param ext: String with the extension name to read
+    :type ext: str
+
+    :return: Dataframe with line labels as index.
+    :rtype: pandas.DataFrame
+
     """
 
     # Fits file:
+    assert Path(lineslog_address).is_file(), f'- Error: lines log not found at {lineslog_address}'
+
     if str(lineslog_address).endswith('.fits'):
         lineslogDF = Table.read(lineslog_address, ext, character_as_bytes=False).to_pandas()
         lineslogDF.set_index('index', inplace=True)
@@ -407,47 +463,51 @@ def load_lines_log(lineslog_address, ext=None):
 
             # Excel file
             try:
-                lineslogDF = pd.read_excel(lineslog_address, sheet_name=0, header=0, index_col=0)
+                lineslogDF = pd.read_excel(lineslog_address, sheet_name=ext, header=0, index_col=0)
             except ValueError:
                 print(f'- ERROR: Could not open lines log at: {lineslog_address}')
 
     return lineslogDF
 
 
-def save_line_log(linelog, file_address, file_type='txt', ext=None, fits_header=None):
+def save_line_log(lines_log, file_address, ext='lineslog', fits_header=None):
+
+    # Confirm file path exits
+    log_path = Path(file_address)
+    assert log_path.parent.exists(), f'- ERROR: Output lines log folder not found ({log_path.parent})'
+    file_name, file_type = log_path.name, log_path.suffix
 
     # Default txt log with the complete information
-    if file_type == 'txt':
-        with open(f'{file_address}.{file_type}', 'wb') as output_file:
-            string_DF = linelog.to_string()
+    if file_type == '.txt':
+        with open(log_path, 'wb') as output_file:
+            string_DF = lines_log.to_string()
             output_file.write(string_DF.encode('UTF-8'))
 
     # Pdf fluxes table
-    elif file_type == 'pdf':
-        table_fluxes(linelog, file_address)
+    elif file_type == '.pdf':
+        table_fluxes(lines_log, log_path.parent/log_path.stem)
 
-    # Linelog in a fits file
-    elif file_type == 'fits':
-        if isinstance(linelog, pd.DataFrame):
-            lineLogHDU = lineslog_to_HDU(linelog, ext_name=ext, header_dict=fits_header)
+    # Lines log in a fits file
+    elif file_type == '.fits':
+        if isinstance(lines_log, pd.DataFrame):
+            lineLogHDU = lineslog_to_HDU(lines_log, ext_name=ext, header_dict=fits_header)
 
-            fits_address = Path(f'{file_address}.{file_type}')
-            if fits_address.is_file():
+            if log_path.is_file():
                 try:
-                    fits.update(fits_address, data=lineLogHDU.data, header=lineLogHDU.header, extname=lineLogHDU.name, verify=True)
+                    fits.update(log_path, data=lineLogHDU.data, header=lineLogHDU.header, extname=lineLogHDU.name, verify=True)
                 except KeyError:
-                    fits.append(fits_address, data=lineLogHDU.data, header=lineLogHDU.header, extname=lineLogHDU.name)
+                    fits.append(log_path, data=lineLogHDU.data, header=lineLogHDU.header, extname=lineLogHDU.name)
             else:
                 hdul = fits.HDUList([fits.PrimaryHDU(), lineLogHDU])
-                hdul.writeto(fits_address, overwrite=True, output_verify='fix')
+                hdul.writeto(log_path, overwrite=True, output_verify='fix')
 
     # Default log in excel format
-    elif file_type == 'xlsx' or file_type == 'xls':
-        sheet_name = ext if ext is not None else 'Sheet1'
-        linelog.to_excel(f'{file_address}.{file_type}', sheet_name=sheet_name)
+    elif file_type == '.xlsx' or file_type == '.xls':
+        sheet_name = ext if ext is not None else 'lineslog'
+        lines_log.to_excel(log_path, sheet_name=ext)
 
     else:
-        print(f"--WARNING: output file extension {file_type} was not recognised. Exiting program")
+        print(f"--WARNING: output extension {file_type} was not recognised in file {log_path}")
         exit()
 
     return
@@ -863,10 +923,185 @@ def spatial_mask_generator(image_flux, mask_param, contour_levels, mask_ref="",
     return
 
 
+def progress_bar(i, i_max, post_text, n_bar=10):
+
+    #size of progress bar
+    j = i/i_max
+    stdout.write('\r')
+    message = f"[{'=' * int(n_bar * j):{n_bar}s}] {int(100 * j)}%  {post_text}"
+    stdout.write(message)
+    stdout.flush()
+
+    return
+
+
+def save_param_maps(log_file_address, param_dict, output_folder, mask_file_address=None, ext_mask='all', image_shape=None,
+                   ext_log='_LINESLOG', default_spaxel_value=np.nan, output_files_prefix=None, page_hdr={}):
+    """
+
+    This function loads a ``.fits`` file with the line log measurements and generates a set of spatial images from a dictionary
+    of parameters and lines provided by the user. For every parameter, the function generates a .fits file with multiple
+    pages (`HDUs <https://docs.astropy.org/en/stable/io/fits/api/hdus.html>`_), one per input line.
+
+    The ``.fits`` log pages will be queried by voxel coordinates (the default format is ``{idx_j}-{idx_i}_LINESLOG``).
+    The user can provide a spatial mask address with the spaxels for which to recover the line log measurements. If the mask
+    ``.fits`` file contains several extensions, the user can provide a list of which ones to use. Otherwise, all will be used.
+
+    .. attention::
+        The user can provide an ``image_shape`` array to generate the output image size. However, in big images attempting this
+        approach, instead of providing a spatial mask with the science data location, can require a long time to inspect
+        the log measurements.
+
+    The output ``.fits`` image maps include a header with the ``PARAM`` and ``LINE`` with the line and parameter labels
+    respectively (see `measurements <documentation/measurements.html>`_).
+
+    :param log_file_address: fits file address location with the line logs
+    :type log_file_address: str
+
+    :param param_dict: Dictionary with the lists of lines to map for every parameter, e.g. ``{'intg_flux': ['O3_5007A', 'H1_6563A']}``
+    :type param_dict: dict
+
+    :param output_folder: Output address for the fits maps
+    :type output_folder: str
+
+    :param mask_file_address: fits file address of the spatial mask images
+    :type mask_file_address: str, optional
+
+    :param ext_mask: Extension or list of extensions in the mask file to determine the list of spaxels to treat. 
+                     By default uses all extensions (special keyword "all") 
+    :type ext_mask: str or list, optional
+
+    :param image_shape: Array with the image spatial size. The unis are the 2D array indices, e.g. (idx_j_max, idx_i_max)
+    :type image_shape: list or array, optional
+
+    :param ext_log: Suffix of the line logs extensions. The default value is "_LINESLOG". In this case the .fits file HDUs
+                    will be queried as ``{idx_j}-{idx_i}_LINESLOG``, where ``idx_j`` and ``idx_i`` are the spaxel Y and X coordinates
+                    respectively
+    :type ext_log: str, optional
+
+    :param default_spaxel_value: Default value for the output image spaxels, where no measurement was obtained from the logs.
+                                 By default this value is numpy.nan
+    :type default_spaxel_value: float, optional
+
+    :param output_files_prefix: Prefix for the output image fits file. e.g. ``f'{output_files_prefix}{parameter}.fits'``. The
+                                default value is None
+    :type output_files_prefix: str, optional
+    
+    :param page_hdr: Dictionary with entries to include in the output parameter HDUs headers
+    :type page_hdr: dict
+
+    :return:
+    """
+
+    assert Path(log_file_address).is_file(), f'- ERROR: lines log at {log_file_address} not found'
+    assert Path(output_folder).is_dir(), f'- ERROR: Output parameter maps folder {output_folder} not found'
+
+    # Compile the list of voxels to recover the provided masks
+    if mask_file_address is not None:
+
+        assert Path(mask_file_address).is_file(), f'- ERROR: mask file at {mask_file_address} not found'
+
+        with fits.open(mask_file_address) as maskHDUs:
+
+            # Get the list of mask extensions
+            if ext_mask == 'all':
+                if ('PRIMARY' in maskHDUs) and (len(maskHDUs) > 1):
+                    mask_list = []
+                    for i, HDU in enumerate(maskHDUs):
+                        mask_name = HDU.name
+                        if mask_name != 'PRIMARY':
+                            mask_list.append(mask_name)
+                    mask_list = np.array(mask_list)
+                else:
+                    mask_list = np.array(['PRIMARY'])
+            else:
+                mask_list = np.array(ext_mask, ndmin=1)
+
+            # Combine all the mask voxels into one
+            for i, mask_name in enumerate(mask_list):
+                if i == 0:
+                    mask_array = maskHDUs[mask_name].data
+                    image_shape = mask_array.shape
+                else:
+                    assert image_shape == maskHDUs[mask_name].data.shape, '- ERROR: Input masks do not have the same dimensions'
+                    mask_array += maskHDUs[mask_name].data
+
+            # Convert to boolean
+            mask_array = mask_array.astype(bool)
+
+            # List of spaxels in list [(idx_j, idx_i), ...] format
+            spaxel_list = np.argwhere(mask_array)
+
+    # No mask file is provided and the user just defines an image size tupple (nY, nX)
+    else:
+        mask_array = np.ones(image_shape).astype(bool)
+        spaxel_list = np.argwhere(mask_array)
+
+    # Generate containers for the data:
+    images_dict = {}
+    for param, line_list in param_dict.items():
+
+        # Make sure is an array and loop throuh them
+        line_list = np.array(line_list, ndmin=1)
+        for line in line_list:
+            images_dict[f'{param}-{line}'] = np.full(image_shape, default_spaxel_value)
+
+    # Loop through the spaxels and fill the parameter images
+    n_spaxels = spaxel_list.shape[0]
+    spaxel_range = np.arange(n_spaxels)
+
+    with fits.open(log_file_address) as logHDUs:
+
+        for i_spaxel in spaxel_range:
+            idx_j, idx_i = spaxel_list[i_spaxel]
+            spaxel_ref = f'{idx_j}-{idx_i}{ext_log}'
+
+            progress_bar(i_spaxel, n_spaxels, postText=f'spaxels treated ({n_spaxels})')
+
+            # Confirm log extension exists
+            if spaxel_ref in logHDUs:
+
+                # Recover extension data
+                log_data = logHDUs[spaxel_ref].data
+                log_lines = log_data['index']
+
+                # Loop through the parameters and the lines:
+                for param, user_lines in param_dict.items():
+                    idcs_log = np.argwhere(np.in1d(log_lines, user_lines))
+                    for i_line in idcs_log:
+                        images_dict[f'{param}-{log_lines[i_line][0]}'][idx_j, idx_i] = log_data[param][i_line][0]
+
+    # New line after the rustic progress bar
+    print()
+
+    # Save the parameter maps as individual fits files with one line per page
+    output_files_prefix = '' if output_files_prefix is None else output_files_prefix
+    for param, user_lines in param_dict.items():
+
+        # Primary header
+        paramHDUs = fits.HDUList()
+        paramHDUs.append(fits.PrimaryHDU())
+
+        # ImageHDU for the parameter maps
+        for line in user_lines:
+            hdr = fits.Header({'PARAM': param, 'LINE': param})
+            hdr.update(page_hdr)
+            data = images_dict[f'{param}-{line}']
+            paramHDUs.append(fits.ImageHDU(name=line, data=data, header=hdr, ver=1))
+
+        # Write to new file
+        output_file = Path(output_folder)/f'{output_files_prefix}{param}.fits'
+        paramHDUs.writeto(output_file, overwrite=True, output_verify='fix')
+
+    return
+
+
 class PdfMaker:
 
     def __init__(self):
+        """
 
+        """
         self.pdf_type = None
         self.pdf_geometry_options = {'right': '1cm',
                                      'left': '1cm',
@@ -878,7 +1113,14 @@ class PdfMaker:
         # TODO add dictionary with numeric formats for tables depending on the variable
 
     def create_pdfDoc(self, pdf_type=None, geometry_options=None, document_class=u'article', theme='white'):
+        """
 
+        :param pdf_type:
+        :param geometry_options:
+        :param document_class:
+        :param theme:
+        :return:
+        """
         # TODO integrate this into the init
         # Case for a complete .pdf or .tex
         self.theme_table = theme
