@@ -26,45 +26,48 @@ def int_to_roman(num):
 def label_decomposition(input_lines, recomb_atom=('H1', 'He1', 'He2'), blended_dict={}, scalar_output=False,
                         user_format={}):
 
-    """This function returns the wavelength, ion and the standard transition latex from the default LiMe line notation.
-    For example:
+    """
+    This function returns the wavelength, ion and the standard transition latex from the default LiMe line notation.
 
     :param input_lines: A string or array of strings with the lime transition notation, e.g. O3_5007A
-    :type input_lines: str
-    :type input_lines: list
+    :type input_lines: str, list
 
     :param recomb_atom: An array with the ions producing photons from a recombination process. By default the function
                         assumes that these are H1, He1 and He2 while the metal ions produce photons from a collisional
                         excited state
-    :type recomb_atom: list, optional
+    :type recomb_atom: str, list
 
-    :param blended_dict: dictionary with the components of the blended lines. For blended lines, the function returns
-                         the ion and wavelength from the first component from the provided list while the scientific
-                         notation appears as the sum of the specified components
+    :param blended_dict: Dictionary with the user latex format for the latex labels, overwritting the default notation.
     :type blended_dict: dict, optional
 
-    :param scalar_output: Boolean for an scalar results in case of a single input line input
+    :param scalar_output: Boolean for a scalar output in case of a single input line input
     :type scalar_output: bool, optional
 
-    :param user_format: Dictionary with the user latex format for the latex labels, overwritting the default notation.
-    :type user_format: dict
+    :param user_format: Dictionary with the user notation for the latex labels. This overwrites the default notation.
+    :type user_format: dict, optional
 
     :return: 3 arrays (or scalars) with the input line(s) transition wavelength, ion and scientific notation in latex
-    :rtype: np.array
+    :rtype: numpy.ndarray
 
     :Example:
-    >>> import lime
-    >>> lime.label_decomposition('O3_5007A', scalar_output=True)
-    O3, 5007.0, '$5007\\AA\\,[OIII]$'
-    >>> lime.label_decomposition('H1_6563A_b', blended_dict={"H1_6563A_b":"H1_6563A-N2_6584A-N2_6548A"})
-    ['H1'], [6563.], ['$6563\\AA\\,HI+6584\\AA\\,[NII]+6548\\AA\\,[NII]$']
+        >>> import lime
+        >>> lime.label_decomposition('O3_5007A', scalar_output=True)
+        O3, 5007.0, '$5007\\AA\\,[OIII]$'
+        >>> lime.label_decomposition('H1_6563A_b', blended_dict={"H1_6563A_b":"H1_6563A-N2_6584A-N2_6548A"})
+        ['H1'], [6563.], ['$6563\\AA\\,HI+6584\\AA\\,[NII]+6548\\AA\\,[NII]$']
+
     """
 
-    # TODO for blended lines it may be better to return all the blended components individually
+
 
     # Confirm input array has one dimension
+    # TODO for blended lines it may be better to return all the blended components individually
     input_lines = np.array(input_lines, ndmin=1)
 
+    # TODO current workflow breaks if repeated labels
+    uniq, count = np.unique(input_lines, return_counts=True)
+    assert not np.any(count > 1), '- ERROR: The input line labels in lime.label_decomposition includes repeated entries, please remove them'
+
     # Containers for input data
     ion_dict, wave_dict, latexLabel_dict = {}, {}, {}
 
@@ -169,7 +172,12 @@ def compute_line_width(idx_peak, spec_flux, delta_i, min_delta=2):
 
 
 def kinematic_component_labelling(line_latex_label, comp_ref):
+    """
 
+    :param line_latex_label:
+    :param comp_ref:
+    :return:
+    """
     if len(comp_ref) != 2:
         print(f'-- Warning: Components label for {line_latex_label} is {comp_ref}. Code only prepare for a 2 character description (ex. n1, w2...)')
 
@@ -200,7 +208,17 @@ class LineFinder:
 
     def match_line_mask(self, mask_df, noise_region, line_threshold=3, emis_abs_thresholds=((4, 4), (1.5, 1.5)),
                         cont_degree_list=(3, 7), width_tol=5, width_mode='auto'):
+        """
 
+        :param mask_df: pandas DataFrame with the line masks.
+        :param noise_region: This region describes the
+        :param line_threshold:
+        :param emis_abs_thresholds:
+        :param cont_degree_list:
+        :param width_tol:
+        :param width_mode:
+        :return:
+        """
         # Convert the observed region to the observed frame
         noise_region_obs = noise_region * (1 + self.redshift)
 
@@ -274,7 +292,6 @@ class LineFinder:
         # Theoretical wave values
         ion_array, waveTheory, latexLabel_array = label_decomposition(matched_DF.index.values)
         matched_DF['wavelength'] = waveTheory
-        # waveTheory = theoLineDF.wavelength.values
 
         # Match the lines with the theoretical emission
         tolerance = np.diff(self.wave_rest).mean() * width_tol
@@ -319,7 +336,6 @@ class LineFinder:
                     matched_DF.loc[row_index, 'w3'] = self.wave_rest[idx_min]
                     matched_DF.loc[row_index, 'w4'] = self.wave_rest[idx_max]
 
-
         # if include_unknown is False:
         #     idcs_unknown = theoLineDF['observation'] == 'not detected'
         #     theoLineDF.drop(index=theoLineDF.loc[idcs_unknown].index.values, inplace=True)
